// Sonnette 1Bit pour ATTiny45
// Son sur PB0
// Front sur PB2

#define F_CPU 8000000UL			// Prvu pour tourner  8Mhz
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <stdlib.h>				// Sert uniquement pour rand()

// Notes:
// 0	Silence
// 2	G#5 	830Hz
// 4	A#5		932Hz
// 1	B5 		987Hz
// 5	D6		1175Hz
// 3	E6 		1318Hz
// 6	F6		1397Hz
// 9	Fin

volatile uint16_t i = 0;		// Compteur de notes
volatile uint8_t temp,tone;		// Indicateurs de mlodie et de la note actuelle.

// Jingle de gurison des pokemons (Gameboy Color)
// Utilise G#5 et B5 et E6
volatile uint8_t tonepokemon [73] = {1,1,1,1,1,1,1,1,1,
							  		 0,0,0,0,0,0,0,0,0,
							  		 1,1,1,1,1,1,1,1,1,
							 		 0,0,0,0,0,0,0,0,0,
									 1,1,1,1,1,1,1,1,1,
									 2,2,2,2,2,2,2,2,2,
									 3,3,3,3,3,3,3,3,3,
									 3,3,3,3,3,3,3,3,3,9};

// 6 premires notes du premier niveau de Super Mario Bros. 1 (NES)
// Utilise A#5, D6 et F6
volatile uint8_t tonemario [55] = {5,5,0,0,5,5,0,0,
								   0,0,0,0,5,5,0,0,
								   0,0,0,0,4,4,0,0,
								   5,5,5,5,0,0,0,0,
								   6,6,6,6,9};

ISR(INT0_vect)
{
}

ISR(TIMER0_OVF_vect)
{
	if (tone == 1)
		temp = tonepokemon[i];
	else
		temp = tonemario[i];
	if(temp != 9) i++;		// Prochaine note.
}

int main(void)
{
	DDRB = 0b11111011;		// Entre sur PB2
	MCUCR = 0x03; 			// Front: 03=montant 02=descendant 01=les deux

	while(1) {
	PORTB &= 0b11111110;	// Rien sur la sortie audio.
	TIMSK = 0x00;
	TCCR0B = 0x00;			// Timer0 teint.
	GIMSK = 0b01000000;
	sei();
	sleep_enable();
	set_sleep_mode(SLEEP_MODE_IDLE);
	sleep_mode();			// Au dodo.
	sleep_disable();
	cli();
	i = 0;
	temp = 0;
	tone = 0; 				// (uint8_t)rand()&1; Pour une mlodie au hasard.
	TIMSK = 0x02;
	TCCR0B = 0x05; 			// Prescaler  1024: 15 interruption par seconde  8Mhz

	while(temp != 9)		// Tant qu'il y a encore des notes  jouer...
	{
		while(temp == 0)
			{
				PORTB &= 0b11111110;	// Silence.
			}
		while(temp == 1)
			{
				PORTB |= 0b00000001;
				_delay_us(506);			// Priode de la note divise par 2.
				PORTB &= 0b11111110;
				_delay_us(506);
			}
		while(temp == 2)
			{
				PORTB |= 0b00000001;
				_delay_us(602);
				PORTB &= 0b11111110;
				_delay_us(602);
			}
		while(temp == 3)
			{
				PORTB |= 0b00000001;
				_delay_us(379);
				PORTB &= 0b11111110;
				_delay_us(379);
			}
		while(temp == 4)
			{
				PORTB |= 0b00000001;
				_delay_us(536);
				PORTB &= 0b11111110;
				_delay_us(536);
			}
		while(temp == 5)
			{
				PORTB |= 0b00000001;
				_delay_us(427);
				PORTB &= 0b11111110;
				_delay_us(427);
			}
		while(temp == 6)
			{
				PORTB |= 0b00000001;
				_delay_us(358);
				PORTB &= 0b11111110;
				_delay_us(358);
			}
		}
	}
}
